<?php
/*
Plugin Name: Fun with Random Comment Forms
Plugin URI: http://www.wp-fun.co.uk/fun-with-random-comment-forms/
Description: Randomises the comment form to prevent automated spam
Author: Andrew Rickmann
Version: 0.2.1
Author URI: http://www.wp-fun.co.uk
Generated At: www.wp-fun.co.uk;
*/ 

/*  Copyright 2008  Andrew Rickmann  (email : mail@arickmann.co.uk)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//make sure the session has started
@session_start();

//what fields do we want to replace?
$fwrc_replaceable_fields = array('author','email','url','comment');

//only run this first bit if we are being included from the comments template
if ( COMMENTS_TEMPLATE === true ){
	global $fw_random_comment_forms;
	
	if ( file_exists( $fw_random_comment_forms->comment_template ) ){
	
		//get the content from the output buffer and process it
		ob_start();
		$comments = get_approved_comments( $post->ID );
		include( $fw_random_comment_forms->comment_template );
		$comment_form = ob_get_contents();
		ob_end_clean();
		
		//Make sure the session is still running after the comment form
		@session_start();
		
		//find and replace form fields
		foreach( $fwrc_replaceable_fields as $replaceable_field ){
			
			//create the value for the field
			$replacement_name = 'fwrcf'.dechex(rand(0,429496729));
			
			//I am going to assume that the name and id fields will be unique
			$comment_form = str_replace('name="'.$replaceable_field.'"' , 'name="'.$replacement_name.'"' , $comment_form );
			$comment_form = str_replace("name='".$replaceable_field."'" , "name='".$replacement_name."'" , $comment_form );
			$comment_form = str_replace('for="'.$replaceable_field.'"' , 'for="'.$replacement_name.'"' , $comment_form );
			$comment_form = str_replace("for='".$replaceable_field."'" , "for='".$replacement_name."'" , $comment_form );			
		
			//save it to the session
			$_SESSION['fwrcf_fields'][$replaceable_field] = $replacement_name;
			
		}
		
		//output the form
		echo $comment_form;		
		
	} else {
		//include the default comment form and exit	
		require( ABSPATH . 'wp-content/themes/default/comments.php');
	}
	
} else {

	if (!class_exists('fw_random_comment_forms')) {
	    class fw_random_comment_forms	{
	
	
			/**
			 * to hold the path of the comment template for later use
			 */
			var $comment_template = '';
			
			
			/**
			* PHP 4 Compatible Constructor
			*/
			function fw_random_comment_forms(){$this->__construct();}
			
			/**
			* PHP 5 Constructor
			*/		
			function __construct(){
	
				//this filter passes the filename of the templatefile
				add_filter( 'comments_template' , array(&$this,'intercept_comments_template'));
				
				//add filter to display the number submissions that bypassed the comment form
				add_action( 'activity_box_end' , array(&$this,'dashboard_report'));
				
				//add the localisation bits
				$fun_with_random_comment_forms_locale = get_locale();
				$fun_with_random_comment_forms_mofile = dirname(__FILE__) . "/languages/fun_with_random_comment_forms-".$fun_with_random_comment_forms_locale.".mo";
				load_textdomain("fun_with_random_comment_forms", $fun_with_random_comment_forms_mofile);				
				
				//carry out the form replacements
				$this->submitted_form_replacements();
			}
			
			/**
			 * 
			 * Echos the number of improper comment requests 
			 */
			function dashboard_report(){
				
				$comment_submit_count = (int) get_option('fwrcf_submit_count');
				
				if ( $comment_submit_count > 0 ) {
					echo '<p>'.sprintf(__('<a href="http://www.wp-fun.co.uk/fun-with-random-comment-forms/" title="Fun with WordPress Plugin Page">Fun with Random Comment Forms</a> has prevented %s automated comments.' , 'fun_with_random_comment_forms'), $comment_submit_count ).'</p>';
				}
				
			}
			
			/**
			 * function check_prerequisits
			 * 
			 * @return false|array 
			 */
			function check_prerequisits(){
				
				//make sure a post is happening
				if ( !isset($_POST) ) { return false; }
				
				//make sure a comment is being submitted
				if ( !isset($_POST['comment_post_ID']) ) { return false; }
				
				//make sure the appropriate session vars are set, if not assume spam, send empty array
				if ( !isset($_SESSION['fwrcf_fields']) ) { return array(); }
				
				//return the field array
				return $_SESSION['fwrcf_fields'];
				
			}
			
			/**
			 * function submitted_form_replacements Replaces the random form field names for the proper names
			 *
			 * @return bool if a real comment has been made
			 */
			function submitted_form_replacements(){
				
				//check prerequisits and get the fields for replacements
				$fields_array = $this->check_prerequisits();
				if ( $fields_array === false  ){ return false; }
								
				//variable to measure essential fields replaced
				$this->essential_replacements = 0;				
								
				foreach( $fields_array as $realName => $codedName ){
							
					//unset the real name
				 	unset( $_POST[$realName] );
							
					//replacement the real name with the codedname, if possible
					if ( isset( $_POST[$codedName] ) ){
						$_POST[$realName] = $_POST[$codedName];
						
						if ( $realName == 'author' || $realName == 'email' || $realName == 'comment' ){
							$this->essential_replacements++;
						}
						
					}
				}
				
				if ( $this->essential_replacements == 3 ){ return true; }
					
				//track the number of times a comment field has been submitted outside the form	
				$option_name = 'fwrcf_submit_count';
									
				if ( $this->essential_replacements == 0 ){
						
					$comment_submit_count = 0;
					
					if ( get_option($option_name) ) {
						$comment_submit_count = get_option($option_name);
						$comment_submit_count +=1;
						update_option( $option_name , $comment_submit_count  );
					} else {
						$deprecated=' ';
						$autoload='no';
						$comment_submit_count +=1;
						add_option($option_name, $comment_submit_count, $deprecated, $autoload);
					}
				} 
				
				//finally, confirm this is not a replacement
				return false;
	
			}
			
			/**
			 * 
			 * @return string The full path of this file
			 * @param string $file The full path of the comment template in the theme folder
			 */
			function intercept_comments_template( $file ){
				$this->comment_template = $file;
				return __FILE__;
			}
	
	    }
	}
	
	//instantiate the class
	if (class_exists('fw_random_comment_forms')) {
		$fw_random_comment_forms = new fw_random_comment_forms();
	}

}


?>